@@ -93,7 +93,7 @@ MpdData * mpd_database_get_artists(MpdObj *mi)
 	{
 		return NULL;
 	}
-	data = mpd_misc_sort_tag_list(data);
+/*	data = mpd_misc_sort_tag_list(data);*/
 	return mpd_data_get_first(data);
 }
 
@@ -169,98 +169,6 @@ MpdData * mpd_database_get_complete(MpdObj *mi)
 	return mpd_data_get_first(data);
 }
 
-MpdData *mpd_database_token_find(MpdObj *mi , char *string)
-{
-	MpdData *data = NULL;
-	mpd_InfoEntity *ent = NULL;
-	regex_t ** strdata = NULL;
-	if(!mpd_check_connected(mi))
-	{
-		debug_printf(DEBUG_WARNING,"not connected\n");
-		return NULL;
-	}
-	if(mpd_lock_conn(mi))
-	{
-		debug_printf(DEBUG_ERROR,"lock failed\n");
-		return NULL;
-	}
-
-	if(string == NULL || !strlen(string) )
-	{
-		debug_printf(DEBUG_INFO, "no string found");
-		mpd_unlock_conn(mi);
-		return NULL;
-	}
-	else{
-		strdata = mpd_misc_tokenize(string);
-	}
-	if(strdata == NULL)
-	{
-		mpd_unlock_conn(mi);
-		debug_printf(DEBUG_INFO, "no split string found");
-		return NULL;
-	}
-
-	mpd_sendListallInfoCommand(mi->connection, "/");
-	while (( ent = mpd_getNextInfoEntity(mi->connection)) != NULL)
-	{
-		if (ent->type == MPD_INFO_ENTITY_TYPE_SONG)
-		{
-			int i = 0;
-			int match = 0;
-			int loop = 1;
-			for(i=0; strdata[i] != NULL && loop; i++)
-			{
-				match = 0;
-				if(ent->info.song->file && !regexec(strdata[i],ent->info.song->file, 0, NULL, 0))
-				{
-					match = 1;
-				}
-				else if(ent->info.song->artist && !regexec(strdata[i],ent->info.song->artist, 0, NULL, 0))
-				{
-					match = 1;
-				}
-				else if(ent->info.song->title && !regexec(strdata[i],ent->info.song->title, 0, NULL, 0)) 
-				{
-					match = 1;
-				}
-				else if(ent->info.song->album && !regexec(strdata[i],ent->info.song->album, 0, NULL, 0))
-				{
-					match = 1;
-				}
-				if(!match)
-				{
-					loop = 0;
-				}
-			}
-
-			if(match)
-			{
-				data = mpd_new_data_struct_append(data);
-				data->type = MPD_DATA_TYPE_SONG;
-				/*
-				data->song = mpd_songDup(ent->info.song);
-				*/
-				data->song = ent->info.song;
-				ent->info.song = NULL;
-			}
-		}
-		mpd_freeInfoEntity(ent);
-	}
-	mpd_finishCommand(mi->connection);
-	mpd_misc_tokens_free(strdata);
-
-
-
-
-	mpd_unlock_conn(mi);
-	if(data == NULL)
-	{
-		return NULL;
-	}
-	return mpd_data_get_first(data);
-}
-
 int mpd_database_delete_playlist(MpdObj *mi,char *path)
 {
 	if(path == NULL)
@@ -377,77 +285,6 @@ MpdData *mpd_misc_sort_tag_list(MpdData *data)
 	return mpd_data_get_first(data);
 }
 
-
-/* TODO need a rewrite */
-/* Should be named: mpd_database_find_adv */
-MpdData *mpd_database_find_adv(MpdObj *mi,int exact, ...)
-{
-	int table = 0;
-	MpdData *data = NULL;
-	mpd_InfoEntity *ent = NULL;
-	va_list arglist;
-	if(!mpd_check_connected(mi))
-	{
-		debug_printf(DEBUG_WARNING,"not connected\n");
-		return NULL;
-	}
-	if(!mpd_server_check_version(mi, 0,12,0))
-	{
-		debug_printf(DEBUG_WARNING, "only works with mpd higher then 0.12.0");
-		return NULL;
-	}
-	if(mpd_lock_conn(mi))
-	{
-		debug_printf(DEBUG_WARNING,"lock failed\n");
-		return NULL;
-	}
-	va_start(arglist, exact);
-	mpd_startSearch(mi->connection, exact);
-	while((table = va_arg(arglist, int)) != -1)
-	{
-		if(table >= 0 && table < MPD_TAG_NUM_OF_ITEM_TYPES)
-		{
-			char *str = va_arg(arglist,char *);
-			mpd_addConstraintSearch(mi->connection, table, str);
-		}	
-	}
-	mpd_commitSearch(mi->connection);
-	va_end(arglist);
-	while (( ent = mpd_getNextInfoEntity(mi->connection)) != NULL)
-	{
-		data = mpd_new_data_struct_append( data );
-		if(ent->type == MPD_INFO_ENTITY_TYPE_DIRECTORY)
-		{
-			data->type = MPD_DATA_TYPE_DIRECTORY;
-			data->directory = ent->info.directory->path;
-			ent->info.directory->path = NULL;
-		}
-		else if (ent->type == MPD_INFO_ENTITY_TYPE_SONG)
-		{
-			data->type = MPD_DATA_TYPE_SONG;
-			data->song = ent->info.song;
-			ent->info.song = NULL;
-		}
-		else if (ent->type == MPD_INFO_ENTITY_TYPE_PLAYLISTFILE)
-		{
-			data->type = MPD_DATA_TYPE_PLAYLIST;
-			data->playlist = ent->info.playlistFile->path;
-			ent->info.playlistFile->path = NULL;
-		}
-
-		mpd_freeInfoEntity(ent);
-	}
-	mpd_finishCommand(mi->connection);
-
-	/* unlock */
-	mpd_unlock_conn(mi);
-	if(data == NULL)
-	{
-		return NULL;
-	}
-	return mpd_data_get_first(data);
-}
-
 /* should be called mpd_database_find */
 MpdData * mpd_database_find(MpdObj *mi, int table, char *string, int exact)
 {
@@ -591,69 +428,6 @@ MpdData * mpd_database_find(MpdObj *mi, int table, char *string, int exact)
 	return mpd_data_get_first(data);
 }
 
-MpdData * mpd_database_get_unique_tags(MpdObj *mi, int field,...)
-{
-	int table = 0;
-	char *string = NULL;
-	MpdData *data = NULL;
-	va_list arglist;
-	if(!mpd_check_connected(mi))
-	{
-		debug_printf(DEBUG_WARNING,"mpd_playlist_get_artists: not connected\n");
-		return NULL;
-	}
-
-	if(!mpd_server_check_version(mi,0,12,0))
-	{
-
-		debug_printf(DEBUG_WARNING, "mpd_playlist_get_unique_tag:For this feature you need at least mpd version 0.12.0");
-		return NULL;
-	}
-	if(table < 0 || table >= MPD_TAG_NUM_OF_ITEM_TYPES)
-	{
-		debug_printf(DEBUG_ERROR, "mpd_playlist_get_unique_tag: Undefined table defined");
-		return NULL;
-	}
-	if(mpd_lock_conn(mi))
-	{
-		debug_printf(DEBUG_WARNING,"mpd_playlist_get_artists: lock failed\n");
-		return NULL;
-	}
-	mpd_startFieldSearch(mi->connection,field);
-	va_start(arglist, field);
-	while((table = va_arg(arglist, int)) != -1)
-	{
-		if(table >= 0 && table < MPD_TAG_NUM_OF_ITEM_TYPES)         	
-		{
-			char *str = va_arg(arglist,char *);
-			mpd_addConstraintSearch(mi->connection, table, str);
-		}	                                                   	
-	}
-	va_end(arglist);
-	mpd_commitSearch(mi->connection);
-	/* TODO check for errors */
-	//	mpd_sendVListTagCommand(mi->connection,table,arglist);
-
-	while (( string = mpd_getNextTag(mi->connection,field)) != NULL)
-	{
-		data = mpd_new_data_struct_append(data);
-		data->type = MPD_DATA_TYPE_TAG;
-		data->tag = string;
-	}
-	/* we need todo this here? can't hurt */
-	mpd_finishCommand(mi->connection);
-	
-	/* unlock */
-	mpd_unlock_conn(mi);
-	if(data == NULL)
-	{
-		return NULL;
-	}
-	data = mpd_misc_sort_tag_list(data);
-	return mpd_data_get_first(data);
-}
-
-
 MpdData * mpd_database_get_directory(MpdObj *mi,char *path)
 {
 	MpdData *data = NULL;
@@ -1027,3 +801,250 @@ MpdData * mpd_database_search_commit(MpdObj *mi)
 	}
 	return mpd_data_get_first(data);
 }
+
+/**
+ * @param mi A #MpdObj
+ *
+ * Starts a search, you can add "constraints" by calling mpd_database_search_add_constraint
+ * to get the result call mpd_database_search_stats_commit
+ * 
+ * This function requires mpd 0.13.0 or higher 
+ */
+
+void mpd_database_search_stats_start(MpdObj *mi)
+{
+	/*
+	 * Check argument
+	 */
+	if(mi == NULL) 
+	{
+		debug_printf(DEBUG_ERROR, "Argument error");
+		return ;
+	}
+	if(!mpd_check_connected(mi))
+	{
+		debug_printf(DEBUG_ERROR, "Not Connected\n");
+		return ;
+	}
+	if(!mpd_server_check_version(mi, 0,13,0))
+	{
+		debug_printf(DEBUG_ERROR, "Advanced search requires mpd 0.13.0 or higher");
+		return ;
+	}
+	/* lock, so we can work on mi->connection */
+	if(mpd_lock_conn(mi) != MPD_OK)
+	{
+		debug_printf(DEBUG_ERROR, "Failed to lock connection");
+		return ;
+	}
+	mpd_startStatsSearch(mi->connection);
+	/* Set search type */
+	mi->search_type = MPD_SEARCH_TYPE_STATS;
+	/* unlock, let the error handler handle any possible error.
+	 */
+	mpd_unlock_conn(mi);
+	return;
+}
+
+MpdDBStats * mpd_database_search_stats_commit(MpdObj *mi)
+{
+	MpdDBStats *data = NULL;
+	if(!mpd_check_connected(mi))
+	{
+		debug_printf(DEBUG_WARNING,"not connected\n");
+		return NULL;
+	}
+	if(mi->search_type != MPD_SEARCH_TYPE_STATS)
+	{
+		debug_printf(DEBUG_ERROR, "no/wrong search in progress to commit");
+		return NULL;
+	}
+	if(mpd_lock_conn(mi))
+	{
+		debug_printf(DEBUG_ERROR,"lock failed\n");
+		return NULL;
+	}
+	mpd_commitSearch(mi->connection);
+
+	data = (MpdDBStats *) mpd_getSearchStats(mi->connection);
+	/* unlock */
+	if(mpd_unlock_conn(mi))
+	{
+		debug_printf(DEBUG_ERROR, "Failed to unlock connection");
+		if(data)mpd_freeSearchStats((mpd_SearchStats *)data);
+		return NULL;
+	}
+	if(data == NULL)
+	{
+		return NULL;
+	}
+	return data;
+}
+
+void mpd_database_search_free_stats(MpdDBStats *data)
+{
+	mpd_freeSearchStats(data);
+}
+
+void mpd_database_playlist_clear(MpdObj *mi, const char *path)
+{
+	if(!path )
+		return;
+	if (!mpd_check_connected(mi)) {
+		debug_printf(DEBUG_WARNING, "not connected\n");
+		return;
+	}
+	if (mpd_status_check(mi) != MPD_OK) {
+		debug_printf(DEBUG_WARNING, "Failed to get status\n");
+		return;
+	}
+	if(mpd_lock_conn(mi))
+	{
+		return ;
+	}
+
+	mpd_sendPlaylistClearCommand(mi->connection, (char *)path);
+	mpd_finishCommand(mi->connection);
+
+	mpd_unlock_conn(mi);
+}
+
+
+
+
+
+
+void mpd_database_playlist_list_delete(MpdObj *mi, const char *path, int pos)
+{
+	if(!path )
+		return;
+	if (!mpd_check_connected(mi)) {
+		debug_printf(DEBUG_WARNING, "not connected\n");
+		return;
+	}
+	if (mpd_status_check(mi) != MPD_OK) {
+		debug_printf(DEBUG_WARNING, "Failed to get status\n");
+		return;
+	}
+	if(mpd_lock_conn(mi))
+	{
+		return ;
+	}
+
+	mpd_sendPlaylistDeleteCommand(mi->connection, (char *)path,pos);
+	mpd_finishCommand(mi->connection);
+
+	mpd_unlock_conn(mi);
+}
+void mpd_database_playlist_list_add(MpdObj *mi, const char *path, const char *file)
+{
+	if(!path )
+		return;
+	if (!mpd_check_connected(mi)) {
+		debug_printf(DEBUG_WARNING, "not connected\n");
+		return;
+	}
+	if (mpd_status_check(mi) != MPD_OK) {
+		debug_printf(DEBUG_WARNING, "Failed to get status\n");
+		return;
+	}
+	if(mpd_lock_conn(mi))
+	{
+		return ;
+	}
+
+	mpd_sendPlaylistAddCommand(mi->connection, (char *)path,(char *)file);
+	mpd_finishCommand(mi->connection);
+
+	mpd_unlock_conn(mi);
+}
+
+MpdData * mpd_database_get_directory_recursive(MpdObj *mi, const char *path)
+{
+	MpdData *data = NULL;
+	mpd_InfoEntity *ent = NULL;
+	if(!mpd_check_connected(mi))
+	{
+		debug_printf(DEBUG_WARNING,"not connected\n");
+		return NULL;
+	}
+	if(path == '\0' || path[0] == '\0')
+	{
+		debug_printf(DEBUG_ERROR, "argumant invalid\n");
+		return NULL;
+	}
+	if(mpd_lock_conn(mi))
+	{
+		debug_printf(DEBUG_ERROR,"lock failed\n");
+		return NULL;
+	}
+	mpd_sendListallInfoCommand(mi->connection,path); 
+	while (( ent = mpd_getNextInfoEntity(mi->connection)) != NULL)
+	{
+
+		if (ent->type == MPD_INFO_ENTITY_TYPE_SONG)
+		{
+			data = mpd_new_data_struct_append(data);
+			data->type = MPD_DATA_TYPE_SONG;
+			data->song = ent->info.song;
+			ent->info.song = NULL;
+		}
+		mpd_freeInfoEntity(ent);
+	}
+	mpd_finishCommand(mi->connection);
+
+	/* unlock */
+	mpd_unlock_conn(mi);
+	if(data == NULL)
+	{
+		return NULL;
+	}
+	return mpd_data_get_first(data);
+}
+void mpd_database_playlist_rename(MpdObj *mi, const char *old, const char *new)
+{
+	if(!new || !old)
+	{
+		debug_printf(DEBUG_ERROR, "old != NULL && new != NULL failed");
+		return;
+	}
+	if (!mpd_check_connected(mi)) {
+		debug_printf(DEBUG_WARNING, "not connected\n");
+		return;
+	}
+	if (mpd_status_check(mi) != MPD_OK) {
+		debug_printf(DEBUG_WARNING, "Failed to get status\n");
+		return;
+	}
+	if(mpd_lock_conn(mi))
+	{
+		return ;
+	}
+
+	mpd_sendRenameCommand(mi->connection, (char *)old,(char *)new);
+	mpd_finishCommand(mi->connection);
+
+	mpd_unlock_conn(mi);
+}
+
+int mpd_database_playlist_move(MpdObj *mi, const char *playlist, int old_pos, int new_pos)
+{
+	if(!mpd_check_connected(mi))
+	{
+		debug_printf(DEBUG_WARNING,"not connected\n");
+		return MPD_NOT_CONNECTED;
+	}
+	if(mpd_lock_conn(mi))
+	{
+		debug_printf(DEBUG_ERROR,"lock failed\n");
+		return MPD_LOCK_FAILED;
+	}
+
+	mpd_sendPlaylistMoveCommand(mi->connection,(char *)playlist,old_pos, new_pos);
+	mpd_finishCommand(mi->connection);
+
+	/* unlock */
+	mpd_unlock_conn(mi);
+	return MPD_OK;
+}
+
