@@ -25,6 +25,7 @@
 #include <regex.h>
 #include <stdarg.h>
 #include <config.h>
+#include <glib.h>
 #include "debug_printf.h"
 
 #include "libmpd.h"
@@ -65,6 +66,8 @@ char * strndup(const char *s, size_t n)
 static void mpd_init_MpdServerState(MpdServerState *state)
 {
 	state->playlistid 	= -1;
+	state->queueid 		= -2;
+	state->storedplaylistid = -1;
 	state->state 		= -1;
 	state->songid 		= -1;     	
 	state->songpos 		= -1;
@@ -80,6 +83,8 @@ static void mpd_init_MpdServerState(MpdServerState *state)
 	state->samplerate 	= 0;
 	state->channels 	= 0;
 	state->bits 		= 0;
+    state->playlistLength = 0;
+    state->error[0]            = '\0';
 
 }
 
@@ -219,10 +224,11 @@ int mpd_check_error(MpdObj *mi)
 	if(mi->error == MPD_ERROR_ACK)
 	{
 
-		debug_printf(DEBUG_ERROR,"clearing errors in mpd_Connection");
+		debug_printf(DEBUG_ERROR,"clearing errors in mpd_Connection: %i-%s", mi->connection->errorCode, mi->connection->errorStr);
 		mpd_clearError(mi->connection);
 		if (mi->the_error_callback)
 		{
+            debug_printf(DEBUG_ERROR, "Error callback 1 (ACK)");
 			mi->the_error_callback(mi, mi->error_mpd_code, mi->error_msg, mi->the_error_signal_userdata );
 		}
 		free(mi->error_msg);
@@ -233,11 +239,13 @@ int mpd_check_error(MpdObj *mi)
 	{
 
 		debug_printf(DEBUG_ERROR, "Following error occured: %i: code: %i msg: %s", mi->error,mi->connection->errorCode, mi->error_msg);
-		mpd_disconnect(mi);
+
 		if (mi->the_error_callback)
 		{
+            debug_printf(DEBUG_ERROR, "Error callback 2");
 			mi->the_error_callback(mi, mi->error, mi->error_msg, mi->the_error_signal_userdata );
 		}
+		mpd_disconnect(mi);
 		free(mi->error_msg);
 		mi->error_msg = NULL;
 
@@ -489,7 +497,10 @@ int mpd_server_get_allowed_commands(MpdObj *mi)
 	}
 	mpd_finishCommand(mi->connection);
 
-	mpd_unlock_conn(mi);
+	if(mpd_unlock_conn(mi))
+    {
+        return MPD_LOCK_FAILED;
+    }
 	return MPD_OK;
 }
 
@@ -497,8 +508,7 @@ int mpd_server_get_allowed_commands(MpdObj *mi)
 
 int mpd_disconnect(MpdObj *mi)
 {
-	/* set disconnect flag */
-	mi->connected = 0;
+
 	/* lock */
 	mpd_lock_conn(mi);
 	debug_printf(DEBUG_INFO, "disconnecting\n");
@@ -524,6 +534,8 @@ int mpd_disconnect(MpdObj *mi)
 		mi->CurrentSong = NULL;
 	}
 	mi->CurrentState.playlistid = -1;
+	mi->CurrentState.queueid = -2;
+	mi->CurrentState.storedplaylistid = -1;
 	mi->CurrentState.state = -1;
 	mi->CurrentState.songid = -1;
 	mi->CurrentState.songpos = -1;
@@ -539,7 +551,8 @@ int mpd_disconnect(MpdObj *mi)
 	mi->CurrentState.samplerate = 0; 
 	mi->CurrentState.channels = 0; 
 	mi->CurrentState.bits = 0;
-
+    mi->CurrentState.playlistLength = 0;
+    mi->CurrentState.error[0] = '\0';
 	/* search stuff */
 	mi->search_type = MPD_SEARCH_TYPE_NONE;
 	/* no need to initialize, but set it to anything anyway*/
@@ -553,18 +566,28 @@ int mpd_disconnect(MpdObj *mi)
 	mpd_free_queue_ob(mi);
 	mpd_server_free_commands(mi);	
 	/*don't reset errors */
-	if(mi->the_connection_changed_callback != NULL)
+	/* Remove this signal, we don't actually disconnect */
+	if(mi->connected)
 	{
-		mi->the_connection_changed_callback( mi, FALSE, mi->the_connection_changed_signal_userdata );
+		/* set disconnect flag */
+		mi->connected = FALSE;
+
+		if(mi->the_connection_changed_callback != NULL)
+		{
+			mi->the_connection_changed_callback( mi, FALSE, mi->the_connection_changed_signal_userdata );
+		}
 	}
-	
 	debug_printf(DEBUG_INFO, "Disconnect completed\n");
 	return MPD_OK;
 }
-
 int mpd_connect(MpdObj *mi)
 {
-	if(mi == NULL)
+	return mpd_connect_real(mi,NULL);
+}
+int mpd_connect_real(MpdObj *mi,mpd_Connection *connection)
+{
+    int retv;
+    if(mi == NULL)
 	{
 		/* should return some spiffy error here */
 		debug_printf(DEBUG_ERROR, "mi != NULL failed");
@@ -599,8 +622,12 @@ int mpd_connect(MpdObj *mi)
 	{
 		mpd_lock_conn(mi);
 	}
-	/* make timeout configurable */
-	mi->connection = mpd_newConnection(mi->hostname,mi->port,mi->connection_timeout);
+	if(connection) {
+		mi->connection = connection;
+	} else {
+		/* make timeout configurable */
+		mi->connection = mpd_newConnection(mi->hostname,mi->port,mi->connection_timeout);
+	}
 	if(mi->connection == NULL)
 	{
 		/* TODO: make seperate error message? */
@@ -620,10 +647,12 @@ int mpd_connect(MpdObj *mi)
 	}
 
 	/* get the commands we are allowed to use */
-	mpd_server_get_allowed_commands(mi);
-
-
-	if(mi->the_connection_changed_callback != NULL)
+	retv = mpd_server_get_allowed_commands(mi);
+    if(retv!= MPD_OK)
+    {
+        return retv;
+    }
+    if(mi->the_connection_changed_callback != NULL)
 	{
 		mi->the_connection_changed_callback( mi, TRUE, mi->the_connection_changed_signal_userdata );
 	}
@@ -680,17 +709,10 @@ void	mpd_signal_connect_connection_changed(MpdObj *mi, ConnectionChangedCallback
 
 /* more playlist */
 /* MpdData Part */
-MpdData *mpd_new_data_struct(MpdData_head * const head)
+MpdData *mpd_new_data_struct(void)
 {
 	MpdData_real* data;
-	if (head->current->space_left == 0) {
-		head->current->next = malloc(sizeof(MpdDataPool));
-		head->current = head->current->next;
-		head->current->space_left = MPD_DATA_POOL_SIZE;
-		head->current->next = NULL;
-	}
-	data = &(head->current->pool[MPD_DATA_POOL_SIZE - head->current->space_left]);
-	head->current->space_left--;
+    data = g_slice_new(MpdData_real);
 	data->type = 0;
 
 	data->tag_type = 0;
@@ -701,30 +723,25 @@ MpdData *mpd_new_data_struct(MpdData_head * const head)
 	data->output_dev = NULL;
 	data->next = NULL;
 	data->prev = NULL;
-	data->head = head;
+    data->first = NULL;
 	return (MpdData*)data;
 }
 
-MpdData *mpd_new_data_struct_append(MpdData  * const data)
+MpdData *mpd_new_data_struct_append(MpdData  * data)
 {
 	MpdData_real *data_real = (MpdData_real*)data;
-	MpdData_head *head;
 	if(data_real == NULL)
 	{
-		head = (MpdData_head*)malloc(sizeof(MpdData_head));
-		head->current = head->pool = (MpdDataPool*)malloc(sizeof(MpdDataPool));
-		head->pool->space_left = MPD_DATA_POOL_SIZE;
-		head->pool->next = NULL;
-		data_real = (MpdData_real*)mpd_new_data_struct(head);
-		head->first = data_real;
+		data_real = (MpdData_real*)mpd_new_data_struct();
+        data_real->first = data_real;
 	}
 	else
 	{
-		data_real->next = (MpdData_real*)mpd_new_data_struct(data_real->head); 	
+		data_real->next = (MpdData_real*)mpd_new_data_struct(); 	
 		data_real->next->prev = data_real;
 		data_real = data_real->next;
 		data_real->next = NULL;
-
+        data_real->first = data_real->prev->first;
 	}
 	return (MpdData*)data_real;
 }
@@ -734,15 +751,8 @@ MpdData * mpd_data_get_first(MpdData const * const data)
 	MpdData_real const * const data_real = (MpdData_real const * const)data;
 	if(data_real != NULL)
 	{
-		if (data_real->head != NULL)
-		{
-			return (MpdData*)(mpd_data_get_head(data)->first);
-		} 
-		else 
-		{
-			return NULL;
-		}
-	}
+        return (MpdData*)data_real->first;
+    }
 	return NULL;
 }
 
@@ -782,18 +792,16 @@ int mpd_data_is_last(MpdData const * const data)
 	}
 	return FALSE;	
 }
-
+/*
 MpdData_head *mpd_data_get_head(MpdData const * const data) {
 	return ((MpdData_real*)data)->head;
 }
-
+*/
 MpdData* mpd_data_concatenate( MpdData  * const first, MpdData  * const second) 
 {
 	MpdData_real *first_real  = (MpdData_real*)first;
 	MpdData_real *second_real = (MpdData_real*)second;
-	MpdData_head *first_head  = NULL;
-	MpdData_head *second_head = NULL; 
-	MpdDataPool *pool;
+	MpdData_real *first_head  = NULL;
 
 	if ( first == NULL ) {
 		if ( second != NULL ) 
@@ -805,8 +813,7 @@ MpdData* mpd_data_concatenate( MpdData  * const first, MpdData  * const second)
 			return (MpdData*)first_real;
 	}
 
-	first_head = mpd_data_get_head(first);
-	second_head = mpd_data_get_head(second);
+	first_head = (MpdData_real *)mpd_data_get_first(first);
 
 	/* find last element in first data list */	
 	while (!mpd_data_is_last((MpdData*)first_real)) first_real = (MpdData_real*)mpd_data_get_next_real((MpdData*)first_real, FALSE);
@@ -818,331 +825,328 @@ MpdData* mpd_data_concatenate( MpdData  * const first, MpdData  * const second)
 	/* I need to set all the -> first correct */
 	while (second_real)
 	{
-		second_real->head = first_head;
+		second_real->first = first_head;
 		second_real = (MpdData_real*)mpd_data_get_next_real((MpdData*)second_real, FALSE);
 	} 
-	/* Need to concatenate the list of MpdDataPools 
-	 * Note that this is NOT efficient in any way...
-	 */
-	pool = first_head->current;
-	pool->next = second_head->pool;
-	first_head->current = second_head->current;
-	free(second_head);
-	return (MpdData*)first_head->first;
+
+	return (MpdData*)first_head;
 }
-/** This function looks broken.  This should be tested
- * Shamefully I don't know this part of the code to well 
+/** 
+ * Deletes an item from the list. It returns the next item in the list.
+ * if that is not available, it will return the last item
  */
 MpdData * mpd_data_delete_item(MpdData *data)
 {
-	MpdData_real *temp = NULL, *data_real = (MpdData_real*)data;
-	if(data_real == NULL) return NULL;
-	if(data_real->head->first == data_real)
-	{
-		/* make temp the first item, and reset the head->first pointer */
-		temp = data_real->head->first = data_real->next;
-		/* there is no item before this. */
-		data_real->prev = NULL;
-		/*  if it was the last item in the list, we need to free the list */
-		if(temp == NULL){
-			mpd_data_free(data);
-		}
-	}
-	else
-	{
+    MpdData_real *temp = NULL, *data_real = (MpdData_real*)data;
+    if(data_real == NULL) return NULL;
+    /* if there is a next item, fix the prev pointer of the next item */
+    if (data_real->next)
+    {
+        data_real->next->prev = data_real->prev;
+        temp = data_real->next;
+    }                                               		
+    /* if there is a previous item, fix the next pointer of the previous item */
+    if (data_real->prev)
+    {
+        /* the next item of the previous is the next item of the current */
+        data_real->prev->next = data_real->next;
+        /* temp is the previous item */
+        temp = data_real->prev;
+    }
 
-		if (data_real->next)
-		{
-			data_real->next->prev = data_real->prev;
-			temp = data_real->next;
-		}                                               		
-		if (data_real->prev)
-		{
-			/* the next item of the previous is the next item of the current */
-			data_real->prev->next = data_real->next;
-			/* temp is the previous item */
-			temp = data_real->prev;
-		}
-	}
+    /* fix first,  if removed item is the first */  
+    if(temp && temp->first == data_real)
+    {
+        MpdData_real *first,*node = temp;
+        /* get first */
+        for(;node->prev;node = node->prev);
+        first = node;
+        while(node){
+            node->first = first;
+            node = node->next;
+        }
+    }
+    /* make the removed row a valid list, so I can use the default free function to free it */
+    data_real->next = NULL;
+    data_real->prev = NULL;
+    data_real->first = data_real;
+    /* free it */
+    mpd_data_free((MpdData *)data_real);
 
-
-	return (MpdData *)temp;
+    return (MpdData *)temp;
 }
 
 void mpd_data_free(MpdData *data)
 {
-	MpdData_head *head;
-	MpdDataPool *pool, *temp_pool;
-	MpdData_real *data_real;
-	unsigned int i;
-	if(data == NULL)
-	{
-		debug_printf(DEBUG_ERROR, "data != NULL Failed");
-		return;
-	}
-	head = mpd_data_get_head(data);
-	pool = head->pool;
-	do {
-		for (i = 0; i < MPD_DATA_POOL_SIZE - pool->space_left; i++) {
-			data_real = &(pool->pool[i]);
-			if (data_real->type == MPD_DATA_TYPE_SONG) {
-				if(data_real->song) mpd_freeSong(data_real->song);
-			} else if (data_real->type == MPD_DATA_TYPE_OUTPUT_DEV) {
-				mpd_freeOutputElement(data_real->output_dev);
-			} else if(data_real->type == MPD_DATA_TYPE_DIRECTORY) {
-				if(data_real->directory)free(data_real->directory);
-			} else if(data_real->type == MPD_DATA_TYPE_PLAYLIST) {
-				if(data_real->playlist)free(data_real->playlist);				
-			} else {
-				free((void*)(data_real->tag));
-			}
-		}
-		temp_pool = pool->next;
-		free (pool);
-		pool = temp_pool;
-	} while ( pool );
-	free(head);
+    MpdData_real *data_real,*temp;
+    if(data == NULL)
+    {
+        debug_printf(DEBUG_ERROR, "data != NULL Failed");
+        return;
+    }
+    data_real = (MpdData_real *)mpd_data_get_first(data);
+    while(data_real){
+        temp = data_real;
+        if (data_real->type == MPD_DATA_TYPE_SONG) {
+            if(data_real->song) mpd_freeSong(data_real->song);
+        } else if (data_real->type == MPD_DATA_TYPE_OUTPUT_DEV) {
+            mpd_freeOutputElement(data_real->output_dev);
+        } else if(data_real->type == MPD_DATA_TYPE_DIRECTORY) {
+            if(data_real->directory)free(data_real->directory);
+        } else if(data_real->type == MPD_DATA_TYPE_PLAYLIST) {
+            if(data_real->playlist)free(data_real->playlist);				
+        } else {
+            free((void*)(data_real->tag));
+        }
+        data_real = data_real->next;
+        g_slice_free1(sizeof(*temp), temp);
+    }
 }
 
 /* clean this up.. make one while loop */
 static void mpd_free_queue_ob(MpdObj *mi)
 {
-	MpdQueue *temp = NULL;
-	if(mi == NULL)
-	{
-		debug_printf(DEBUG_ERROR, "mi != NULL failed");
-		return;
-	}
-	if(mi->queue == NULL)
-	{
-		debug_printf(DEBUG_INFO, "mi->queue != NULL failed, nothing to clean.");
-		return;
-	}	
-	mi->queue = mi->queue->first;
-	while(mi->queue != NULL)
-	{
-		temp = mi->queue->next;
+    MpdQueue *temp = NULL;
+    if(mi == NULL)
+    {
+        debug_printf(DEBUG_ERROR, "mi != NULL failed");
+        return;
+    }
+    if(mi->queue == NULL)
+    {
+        debug_printf(DEBUG_INFO, "mi->queue != NULL failed, nothing to clean.");
+        return;
+    }	
+    mi->queue = mi->queue->first;
+    while(mi->queue != NULL)
+    {
+        temp = mi->queue->next;
 
-		if(mi->queue->path != NULL)
-		{
-			free(mi->queue->path);
-		}
+        if(mi->queue->path != NULL)
+        {
+            free(mi->queue->path);
+        }
 
-		free(mi->queue);
-		mi->queue = temp;
-	}
-	mi->queue = NULL;
+        free(mi->queue);
+        mi->queue = temp;
+    }
+    mi->queue = NULL;
 
 }
 
 MpdQueue *mpd_new_queue_struct()
 {
-	MpdQueue* queue = malloc(sizeof(MpdQueue));
+    MpdQueue* queue = malloc(sizeof(MpdQueue));
 
-	queue->type = 0;
-	queue->path = NULL;
-	queue->id = 0;
+    queue->type = 0;
+    queue->path = NULL;
+    queue->id = 0;
 
-	return queue;	
+    return queue;	
 }
 
 
 void mpd_queue_get_next(MpdObj *mi)
 {
-	if(mi->queue != NULL && mi->queue->next != NULL)
-	{
-		mi->queue = mi->queue->next;
-	}
-	else if(mi->queue->next == NULL)
-	{
-		mpd_free_queue_ob(mi);
-		mi->queue = NULL;
-	}
+    if(mi->queue != NULL && mi->queue->next != NULL)
+    {
+        mi->queue = mi->queue->next;
+    }
+    else if(mi->queue->next == NULL)
+    {
+        mpd_free_queue_ob(mi);
+        mi->queue = NULL;
+    }
 }
 
 long unsigned mpd_server_get_database_update_time(MpdObj *mi)
 {
-	if(!mpd_check_connected(mi))
-	{
-		debug_printf(DEBUG_WARNING,"not connected\n");
-		return MPD_NOT_CONNECTED;
-	}
-	if(mpd_stats_check(mi) != MPD_OK)
-	{
-		debug_printf(DEBUG_WARNING,"Failed grabbing status\n");
-		return MPD_STATS_FAILED;
-	}
-	return mi->stats->dbUpdateTime;
+    if(!mpd_check_connected(mi))
+    {
+        debug_printf(DEBUG_WARNING,"not connected\n");
+        return MPD_NOT_CONNECTED;
+    }
+    if(mpd_stats_check(mi) != MPD_OK)
+    {
+        debug_printf(DEBUG_WARNING,"Failed grabbing status\n");
+        return MPD_STATS_FAILED;
+    }
+    return mi->stats->dbUpdateTime;
 }
 
 
 MpdData * mpd_server_get_output_devices(MpdObj *mi)
 {
-	mpd_OutputEntity *output = NULL;
-	MpdData *data = NULL;
-	if(!mpd_check_connected(mi))
-	{
-		debug_printf(DEBUG_WARNING,"not connected\n");
-		return NULL;
-	}
-	/* TODO: Check version */
-	if(mpd_lock_conn(mi))
-	{
-		debug_printf(DEBUG_ERROR,"lock failed\n");
-		return NULL;
-	}
+    mpd_OutputEntity *output = NULL;
+    MpdData *data = NULL;
+    if(!mpd_check_connected(mi))
+    {
+        debug_printf(DEBUG_WARNING,"not connected\n");
+        return NULL;
+    }
+    /* TODO: Check version */
+    if(mpd_lock_conn(mi))
+    {
+        debug_printf(DEBUG_ERROR,"lock failed\n");
+        return NULL;
+    }
 
-	mpd_sendOutputsCommand(mi->connection);
-	while (( output = mpd_getNextOutput(mi->connection)) != NULL)
-	{	
-		data = mpd_new_data_struct_append(data);
-		data->type = MPD_DATA_TYPE_OUTPUT_DEV; 
-		data->output_dev = output;
-	}
-	mpd_finishCommand(mi->connection);
+    mpd_sendOutputsCommand(mi->connection);
+    while (( output = mpd_getNextOutput(mi->connection)) != NULL)
+    {	
+        data = mpd_new_data_struct_append(data);
+        data->type = MPD_DATA_TYPE_OUTPUT_DEV; 
+        data->output_dev = output;
+    }
+    mpd_finishCommand(mi->connection);
 
-	/* unlock */
-	mpd_unlock_conn(mi);
-	if(data == NULL) 
-	{
-		return NULL;
-	}
-	return mpd_data_get_first(data);
+    /* unlock */
+    mpd_unlock_conn(mi);
+    if(data == NULL) 
+    {
+        return NULL;
+    }
+    return mpd_data_get_first(data);
 }
 
 int mpd_server_set_output_device(MpdObj *mi,int device_id,int state)
 {
-	if(!mpd_check_connected(mi))
-	{
-		debug_printf(DEBUG_WARNING,"not connected\n");	
-		return MPD_NOT_CONNECTED;
-	}
-	if(mpd_lock_conn(mi))
-	{
-		debug_printf(DEBUG_ERROR,"lock failed\n");
-		return MPD_LOCK_FAILED;
-	}
-	if(state)
-	{
-		mpd_sendEnableOutputCommand(mi->connection, device_id);
-	}
-	else
-	{
-		mpd_sendDisableOutputCommand(mi->connection, device_id);
-	}	
-	mpd_finishCommand(mi->connection);
+    if(!mpd_check_connected(mi))
+    {
+        debug_printf(DEBUG_WARNING,"not connected\n");	
+        return MPD_NOT_CONNECTED;
+    }
+    if(mpd_lock_conn(mi))
+    {
+        debug_printf(DEBUG_ERROR,"lock failed\n");
+        return MPD_LOCK_FAILED;
+    }
+    if(state)
+    {
+        mpd_sendEnableOutputCommand(mi->connection, device_id);
+    }
+    else
+    {
+        mpd_sendDisableOutputCommand(mi->connection, device_id);
+    }	
+    mpd_finishCommand(mi->connection);
 
-	mpd_unlock_conn(mi);
-	mpd_status_queue_update(mi);
-	return FALSE;
+    mpd_unlock_conn(mi);
+    mpd_status_queue_update(mi);
+    return FALSE;
 }
 
 int mpd_server_check_version(MpdObj *mi, int major, int minor, int micro)
 {
-	if(!mpd_check_connected(mi))
-	{
-		debug_printf(DEBUG_WARNING,"not connected\n");
-		return FALSE;
-	}
-	if(major > mi->connection->version[0]) return FALSE;
-	if(mi->connection->version[0] > major) return TRUE;
-	if(minor > mi->connection->version[1]) return FALSE;
-	if(mi->connection->version[1] > minor) return TRUE;
-	if(micro > mi->connection->version[2]) return FALSE;
-	if(mi->connection->version[2] > micro) return TRUE; 	
-	return TRUE;
+    if(!mpd_check_connected(mi))
+    {
+        debug_printf(DEBUG_WARNING,"not connected\n");
+        return FALSE;
+    }
+    if(major > mi->connection->version[0]) return FALSE;
+    if(mi->connection->version[0] > major) return TRUE;
+    if(minor > mi->connection->version[1]) return FALSE;
+    if(mi->connection->version[1] > minor) return TRUE;
+    if(micro > mi->connection->version[2]) return FALSE;
+    if(mi->connection->version[2] > micro) return TRUE; 	
+    return TRUE;
 }	
 
 int mpd_server_check_command_allowed(MpdObj *mi, const char *command)
 {
-	int i;
-	if(!mi || !command) return MPD_SERVER_COMMAND_ERROR;
-	/* when we are connected to a mpd server that doesn't support commands and not commands
-	 * feature. (like mpd 0.11.5) allow everything
-	 */
-	if(!mpd_server_check_version(mi, 0,12,0)) return MPD_SERVER_COMMAND_ALLOWED;
-	/*
-	 * Also when somehow we failted to get commands
-	 */
-	if(mi->commands == NULL) return MPD_SERVER_COMMAND_ALLOWED;
+    int i;
+    if(!mi || !command) return MPD_SERVER_COMMAND_ERROR;
+    /* when we are connected to a mpd server that doesn't support commands and not commands
+     * feature. (like mpd 0.11.5) allow everything
+     */
+    if(!mpd_server_check_version(mi, 0,12,0)) return MPD_SERVER_COMMAND_ALLOWED;
+    /*
+     * Also when somehow we failted to get commands
+     */
+    if(mi->commands == NULL) return MPD_SERVER_COMMAND_ALLOWED;
 
 
 
-	for(i=0;mi->commands[i].command_name;i++)
-	{
-		if(!strcasecmp(mi->commands[i].command_name, command))
-			return mi->commands[i].enabled;
-	}
-	return MPD_SERVER_COMMAND_NOT_SUPPORTED;
+    for(i=0;mi->commands[i].command_name;i++)
+    {
+        if(!strcasecmp(mi->commands[i].command_name, command))
+            return mi->commands[i].enabled;
+    }
+    return MPD_SERVER_COMMAND_NOT_SUPPORTED;
 }
 
-/** MISC **/
-
-regex_t ** mpd_misc_tokenize(char *string)
+char ** mpd_server_get_url_handlers(MpdObj *mi)
 {
-	regex_t ** result = NULL; 	/* the result with tokens 		*/
-	int i = 0;		/* position in string 			*/
-	int br = 0;		/* number for open ()[]'s		*/
-	int bpos = 0;		/* begin position of the cur. token 	*/
+    char *temp = NULL;
+    int i=0;
+    char **retv = NULL;
+    if(!mpd_check_connected(mi))
+    {
+        debug_printf(DEBUG_WARNING,"not connected\n");
+        return FALSE;
+    }
+    if(mpd_lock_conn(mi))
+    {
+        debug_printf(DEBUG_ERROR,"lock failed\n");
+        return NULL;
+    }                                           
+    mpd_sendUrlHandlersCommand(mi->connection);
+    while((temp = mpd_getNextHandler(mi->connection)) != NULL)
+    {
+        retv = realloc(retv,(i+2)*sizeof(*retv));
+        retv[i]   = temp;
+        retv[i+1] = NULL;
+        i++;
+    } 
+    mpd_finishCommand(mi->connection);
 
-	int tokens=0;
-	if(string == NULL) return NULL;
-	for(i=0; i < strlen(string)+1;i++)
-	{
-		/* check for opening  [( */
-		if(string[i] == '(' || string[i] == '[' || string[i] == '{') br++;
-		/* check closing */
-		else if(string[i] == ')' || string[i] == ']' || string[i] == '}') br--;
-		/* if multiple spaces at begin of token skip them */
-		else if(string[i] == ' ' && !(i-bpos))bpos++;
-		/* if token end or string end add token to list */
-		else if((string[i] == ' ' && !br) || string[i] == '\0')
-		{
-			char * temp=NULL;
-			result = (regex_t **)realloc(result,(tokens+2)*sizeof(regex_t *));
-			result[tokens] = malloc(sizeof(regex_t));
-			temp = (char *)strndup((const char *)&string[bpos], i-bpos);
-			if(regcomp(result[tokens], temp, REG_EXTENDED|REG_ICASE|REG_NOSUB))
-			{
-				result[tokens+1] = NULL;
-				mpd_misc_tokens_free(result);
-				return NULL;
-			}
-			free(temp);
-			result[tokens+1] = NULL;
-			bpos = i+1;
-			tokens++;
-		}
 
-	}
-	return result;
+    mpd_unlock_conn(mi);
+    return retv;
 }
-
-void mpd_misc_tokens_free(regex_t ** tokens)
+char ** mpd_server_get_tag_types(MpdObj *mi)
 {
-	int i=0;
-	if(tokens == NULL) return;
-	for(i=0;tokens[i] != NULL;i++)
-	{
-		regfree(tokens[i]);
-		free(tokens[i]);
-	}
-	free(tokens);
+    char *temp = NULL;
+    int i=0;
+    char **retv = NULL;
+    if(!mpd_check_connected(mi))
+    {
+        debug_printf(DEBUG_WARNING,"not connected\n");
+        return FALSE;
+    }
+    if(mpd_lock_conn(mi))
+    {
+        debug_printf(DEBUG_ERROR,"lock failed\n");
+        return NULL;
+    }                                           
+    mpd_sendTagTypesCommand(mi->connection);
+    while((temp = mpd_getNextTagType(mi->connection)) != NULL)
+    {
+        retv = realloc(retv,(i+2)*sizeof(*retv));
+        retv[i]   = temp;
+        retv[i+1] = NULL;
+        i++;
+    } 
+    mpd_finishCommand(mi->connection);
+
+
+    mpd_unlock_conn(mi);
+    return retv;
 }
 
 int mpd_misc_get_tag_by_name(char *name)
 {
-	int i;
-	if(name == NULL)
-	{
-		return MPD_ARGS_ERROR;
-	}
-	for(i=0; i < MPD_TAG_NUM_OF_ITEM_TYPES; i++)
-	{
-		if(!strcasecmp(mpdTagItemKeys[i], name))
-		{
-			return i;
-		}
-	}
-	return MPD_TAG_NOT_FOUND;
+    int i;
+    if(name == NULL)
+    {
+        return MPD_ARGS_ERROR;
+    }
+    for(i=0; i < MPD_TAG_NUM_OF_ITEM_TYPES; i++)
+    {
+        if(!strcasecmp(mpdTagItemKeys[i], name))
+        {
+            return i;
+        }
+    }
+    return MPD_TAG_NOT_FOUND;
 }
