@@ -41,6 +41,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <fcntl.h>
+#include <limits.h>
 
 #ifdef WIN32
 #  include <ws2tcpip.h>
@@ -672,6 +673,8 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 	status->repeat = 0;
 	status->random = 0;
 	status->playlist = -1;
+	status->playlistqueue = -1;
+	status->storedplaylist = -1;
 	status->playlistLength = -1;
 	status->state = -1;
 	status->song = 0;
@@ -704,6 +707,12 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 		else if(strcmp(re->name,"playlist")==0) {
 			status->playlist = strtol(re->value,NULL,10);
 		}
+		else if(strcmp(re->name,"playlistqueue")==0) {
+			status->playlistqueue = strtol(re->value,NULL,10);
+		}
+		else if (strcmp(re->name, "storedplaylist") == 0 ) {
+			status->storedplaylist = strtol(re->value,NULL,10);
+		}
 		else if(strcmp(re->name,"playlistlength")==0) {
 			status->playlistLength = atoi(re->value);
 		}
@@ -1423,6 +1432,20 @@ void mpd_sendRmCommand(mpd_Connection * connection, const char * name) {
 	free(sName);
 }
 
+void mpd_sendRenameCommand(mpd_Connection *connection, const char *from,
+                           const char *to)
+{
+	char *sFrom = mpd_sanitizeArg(from);
+	char *sTo = mpd_sanitizeArg(to);
+	int len = strlen("rename")+2+strlen(sFrom)+3+strlen(sTo)+3;
+	char *string = malloc(len);
+	snprintf(string, len, "rename \"%s\" \"%s\"\n", sFrom, sTo);
+	mpd_executeCommand(connection, string);
+	free(string);
+	free(sFrom);
+	free(sTo);
+}
+
 void mpd_sendShuffleCommand(mpd_Connection * connection) {
 	mpd_executeCommand(connection,"shuffle\n");
 }
@@ -1715,7 +1738,7 @@ char * mpd_getNextCommand(mpd_Connection * connection)
 
 void mpd_sendUrlHandlersCommand(mpd_Connection * connection)
 {
-	mpd_executeCommand(connection, "urlhandlers");
+	mpd_executeCommand(connection, "urlhandlers\n");
 }
 
 char * mpd_getNextHandler(mpd_Connection * connection)
@@ -1725,7 +1748,7 @@ char * mpd_getNextHandler(mpd_Connection * connection)
 
 void mpd_sendTagTypesCommand(mpd_Connection * connection)
 {
-	mpd_executeCommand(connection, "tagtypes");
+	mpd_executeCommand(connection, "tagtypes\n");
 }
 
 char * mpd_getNextTagType(mpd_Connection * connection)
@@ -1938,3 +1961,25 @@ void mpd_sendPlaylistDeleteCommand(mpd_Connection *connection,
 	free(sPlaylist);
 	free(string);
 }
+
+void mpd_sendQueueInfoCommand(mpd_Connection * connection) {
+	mpd_sendInfoCommand(connection,"queueinfo\n");
+}
+
+void mpd_sendDequeueCommand(mpd_Connection * connection, int songPos) {
+	int len = strlen("dequeue")+2+INTLEN+3;
+	char *string = malloc(len);
+	snprintf(string, len, "dequeue \"%i\"\n", songPos);
+	mpd_executeCommand(connection,string);
+	free(string);
+}
+void mpd_sendQueueIdCommand(mpd_Connection * connection, int songId) {
+	int len = strlen("queueid")+2+INTLEN+3;
+	char *string = malloc(len);
+	snprintf(string, len, "queueid \"%i\"\n", songId);
+	mpd_executeCommand(connection,string);
+	free(string);
+}
+void mpd_sendClearErrorCommand(mpd_Connection * connection) {
+	mpd_executeCommand(connection,"clearerror\n");
+}
